//----------------------------------------------------------------------------------------
// UJavelinApplication.cp
// Copyright 1991-1992 Apple Computer, Inc.
//----------------------------------------------------------------------------------------

#ifndef __UAPPLICATIONJAVELIN__
#include "UApplicationJavelin.h"
#endif

#ifndef __UDOCUMENTJAVELIN__
#include "UDocumentJavelin.h"
#endif

#ifndef __UCOMMANDJAVELIN__
#include "UCommandJavelin.h"
#endif

#ifndef __UREMOTECOMMANDS__
#include "URemoteCommands.h"
#endif

#ifndef __AUXAESuite__
#include "AUXAESuite.h"
#endif

#ifndef TOOLUTILS
#include <ToolUtils.h>
#endif

#ifndef PACKAGES
#include <Packages.h>
#endif

//========================================================================================
// CLASS TApplicationJavelin
//========================================================================================

//----------------------------------------------------------------------------------------
// TApplicationJavelin::IApplicationJavelin: 
//----------------------------------------------------------------------------------------
#pragma segment AInit
pascal void TApplicationJavelin::IApplicationJavelin()
{
	this->IApplication(kSignature,kSignature);
	TIdleBeeper* myIdleBeeper = new TIdleBeeper;
	myIdleBeeper->IIdleBeeper();
	
} // TApplicationJavelin::IApplicationJavelin 


//----------------------------------------------------------------------------------------
// TApplicationJavelin::DoMakeDocument: This method is overridden to return a document
// object of the appropriate class for this application. MacApp calls this method when the
// user chooose New or Open from the File menu. For applications which hande multiple
// document types, the command number can be used to discriminate between different user
// requests. The file object represents the file chosen by the user in the Standard File
// dialog.
//----------------------------------------------------------------------------------------
#pragma segment AOpen		
pascal TDocument* TApplicationJavelin::DoMakeDocument(CommandNumber /*itsCommandNumber*/,
														TFile* /*itsFile*/) // Override 
{
	TDocumentJavelin* aDocument = new TDocumentJavelin;
	
	aDocument->IDocumentJavelin();
	return aDocument;
} // TApplicationJavelin::DoMakeDocument 

//----------------------------------------------------------------------------------------
// TApplicationJavelin::DoMenuCommand: This method is overridden to handle menu items
// which are enabled when this object is in the target chain. The application object is
// always in the target chain.
// 
// The inherited method is called so that MacApp can handle application-level menu items
// like the "About " menu item in the Apple menu.
//----------------------------------------------------------------------------------------
#pragma segment ASelCommand
pascal void TApplicationJavelin::DoMenuCommand(CommandNumber aCommandNumber) // Override 
{
	switch (aCommandNumber) 
	{
		case cCommandHandledByApplication :
			SysBeep(2);
			break;
		default:
			inherited::DoMenuCommand(aCommandNumber);
			break;
	}
} // TApplicationJavelin::DoMenuCommand 

//----------------------------------------------------------------------------------------
// TApplicationJavelin::DoSetupMenus: This method is overridden to enable menu items
// which should be enabled when this object is in the target chain. MacApp initially
// disables all menu items, then lets the objects in the target chain enable those items
// they handle.
//
// The application object is always in the target chain, so the About menu item in the
// Apple menu and the item with command number cMenuHandledByApplication should be enabled
// even if there are no open documents.
// 
// The inherited method is called so that MacApp can enable application-level menu items
// like the "About " menu item.
//----------------------------------------------------------------------------------------
#pragma segment ARes
pascal void TApplicationJavelin::DoSetupMenus() // Override 
{
	inherited::DoSetupMenus();				// Always call the inherited method first
	Enable(cCommandHandledByApplication, !MemSpaceIsLow());

} // TApplicationJavelin::DoSetupMenus 


//--------------------------------------------------------------------------------------------------
#pragma segment ARes
pascal void TApplicationJavelin::DoAppleCommand(CommandNumber aCommandNumber,
												const AppleEvent& message,
												const AppleEvent& reply)  // override
{

	long actualSize;
	int statusCode;
	char string[1024];
	DescType actualCode;
	CStr255 theString = gEmptyString;
	long theSessionID = 0;
	TDocumentJavelin* aDoc;	 

switch (aCommandNumber) {
	case cAEHeartBeat:

		FailOSErr(AEGetParamPtr(message, keySessionID, typeLongInteger, actualCode, (Ptr) & theSessionID, sizeof(theSessionID), actualSize));
		aDoc = (TDocumentJavelin*)this->LocateDocument(theSessionID);
		if (aDoc != NULL)
			aDoc->HaveAHeartBeat();

		break;
	case cAEJavMsg:
		FailOSErr(AEGetParamPtr(message, keySessionID, typeLongInteger, actualCode, (Ptr) & theSessionID, sizeof(theSessionID), actualSize));
		FailOSErr(AEGetParamPtr(message, keyMessageCode, typeInteger, actualCode,(Ptr)&statusCode, sizeof(int), actualSize));
		
		aDoc = (TDocumentJavelin*)this->LocateDocument(theSessionID);
		if (aDoc != NULL)
			aDoc->CommandCompleted();  //either way I close this puppy
		if (statusCode == 0)
			{
			AEGetParamPtr(message, keyMsgString, typeChar, actualCode,(Ptr)string, 1024, actualSize);
	
			string[actualSize] = NULL;
		
			// Wierdness for converting AE/UNIX strings to Mac strings.
			for (char* ptr = string; *ptr != NULL; ptr++)
				if (*ptr == '\r')
					*ptr = '\n';
					
			theString = string;
			aDoc->SetWindowText(theString);
			return;
			}
		GetIndString(theString,kAlertUserToError  /*1001*/, statusCode);

//optional string supplied by the server


		AEGetParamPtr(message, keyMsgString, typeChar, actualCode,(Ptr)string, 1024, actualSize);

		string[actualSize] = NULL;
	
		// Wierdness for converting AE/UNIX strings to Mac strings.
		for (char* ptr = string; *ptr != NULL; ptr++)
			if (*ptr == '\r')
				*ptr = '\n';
				
		theString = string;
		
		JavelinErrorBox(theString);
		
		break;
	default:
		{
		inherited::DoAppleCommand(aCommandNumber, message, reply);
		}
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment ARes
pascal TDocumentJavelin* TApplicationJavelin::LocateDocument(long theSessionID)
{
	CDocumentIterator iter(this);

	for (TDocument* aDocument = iter.FirstDocument(); iter.More(); aDocument = iter.NextDocument())
		if (((TDocumentJavelin*)aDocument)->LocateDocument(theSessionID))
			return (TDocumentJavelin*) aDocument;
	return NULL;								// No document already open
}
//--------------------------------------------------------------------------------------------------
#pragma segment ARes
pascal Boolean TApplicationJavelin::IsThisMyDoc(TDocument* theDoc)
{
	CDocumentIterator iter(this);
	if (IsObject(theDoc) && (iter.FirstDocument() != NULL))
		{
		if (GetClassID(theDoc) == GetClassID(iter.FirstDocument()))
			{
			for (TDocument* aDocument = iter.FirstDocument(); iter.More(); aDocument = iter.NextDocument())
				if (aDocument == theDoc)   //compare pointers right?
					return TRUE;
			}
		}
		
	return FALSE;								// No document already open
}

//--------------------------------------------------------------------------------------------------
#pragma segment ANonRes
pascal void ptocstr(char* string)
{
	register char *to = string;
	register char length = *string++, i;

	for (i = 1; i <= length; i++)
		*to++ = *string++;
	*to = '\0';
}

//==================================================================================================
#pragma segment AIdle

void pascal TCohandler::Free()
/*
Task:
	Remove me as Cohandler before freeing.
*/
{
	// remove as cohandler of application object
	gApplication->InstallCohandler(this, false);

	inherited::Free();
}
//--------------------------------------------------------------------------------------------------
#pragma segment AIdle

void pascal TCohandler::ICohandler()
/*
Task:
	Add me as Cohandler.
*/
{
	// install as cohandler of application object
	this->IEventHandler(NULL);
	gApplication->InstallCohandler(this, true);
}
//==================================================================================================
#pragma segment AIdle

void pascal TIdleBeeper::IIdleBeeper()
{
	this->ICohandler();
	fIdleFreq = 1800;   //30 seconds minimum
	fLastHeartBeat = TickCount()/60;
}

//--------------------------------------------------------------------------------------------------
#pragma segment AIdle

Boolean pascal TIdleBeeper::DoIdle(IdlePhase)
{
	if (((TickCount()/60) - fLastHeartBeat) > 30)
		{
		fLastHeartBeat = TickCount()/60;

		CDocumentIterator iter(gApplication);
//		THeartBeatCommand* aCommand = new THeartBeatCommand;

		for (TDocument* aDocument = iter.FirstDocument(); iter.More(); aDocument = iter.NextDocument())
			{
			THeartBeatCommand* aCommand = new THeartBeatCommand;
			aCommand->IHeartBeatCommand((TDocumentJavelin*)aDocument);
 			((TDocumentJavelin*)aDocument)->PostCommand(aCommand);
			if (! ((TDocumentJavelin*)aDocument)->CheckPulse())            //he's dead Jim!
				((TDocumentJavelin*)aDocument)->CloseAndFree();
			}
		}

	return false;  //doesn't matter I guess
}
