// URemoteCommands.cp
// Copyright 1991-1992 Apple Computer, Inc.


#ifndef __UDOCUMENTJAVELIN__
#include "UDocumentJavelin.h"
#endif

#ifndef __AUXAESUITE__
#include "AUXAESuite.h"
#endif

#ifndef __UCOMMANDJAVELIN__
#include "UCommandJavelin.h"
#endif

#ifndef __UREMOTECOMMANDS__
#include "URemoteCommands.h"
#endif

#ifndef __JAVELINAPPLEEVENT__
#include "UJavelinAppleEvent.h"
#endif

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __STDIO__
#include <stdIO.h>
#endif

#ifndef __STDLIB__
#include <StdLib.h>
#endif

#ifndef __STRING__
#include <String.h>
#endif



//--------------------------------------------------------------------------------------------------
// superclasss for  client commands
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
#pragma segment ACommand
pascal TAppleEvent* TJavelinClientCommand::SendMessage()
{
	((TDocumentJavelin*)fContext)->PendingCommand();
	return inherited::SendMessage();
}



//--------------------------------------------------------------------------------------------------
#pragma segment ACommand
pascal void TJavelinClientCommand::IJavelinClientCommand(TDocumentJavelin* itsDocument,
														const int theKey,
														TObject* toNotify)
{
	AEAddressDesc theAddress;

	this->IClientCommand(cRemoteCommand, itsDocument, kCantUndo, kCausesChange, toNotify);

	FailInfo fi;
	if (fi.Try())
	{
		itsDocument->GetAEAddressDesc(theAddress);

		TJavelinAppleEvent* aMessage = new TJavelinAppleEvent;		// create the message.
		
		aMessage->IJavelinAppleEvent(kAECmdSuiteID, (AEEventID)theKey, theAddress, kAEQueueReply, itsDocument->GetPortRefNum() );
		fMessage = aMessage;
		fi.Success();
	}
	else
	{
		this->Free();
		fi.ReSignal();
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment ACommand
pascal void TJavelinClientCommand::ProcessReply(TAppleEvent* theReply)
{
	char status[1];
	char string[1024];
	long actualSize;
	DescType actualCode;

	inherited::ProcessReply(theReply);
	
	AppleEvent theMessage = theReply->fMessage;

	OSErr err = AEGetParamPtr(theMessage, keyErrorNumber, typeChar, actualCode, (Ptr)status, sizeof(char), actualSize);

	if (err != noErr)
		{
		((TDocumentJavelin*)fContext)->CommandCompleted();   //so we're not hosed completely
		CStr255 theError;
		theError = "Host is not sending F or S in the keyErrorNumber Desc in the last AE that expected a reply. Help Mr. Wizard!";
		JavelinErrorBox(theError);
		return;
		}

	if (*status == 'S') 
		{
		return;
		}
					
	((TDocumentJavelin*)fContext)->CommandCompleted();
	
	theReply->ReadParameterPtr(keyErrorString, typeChar, actualCode,(Ptr)string, 1024, actualSize);
	string[actualSize] = NULL;

	// Wierdness for converting AE/UNIX strings to Mac strings.
	for (char* ptr = string; *ptr != NULL; ptr++)
		if (*ptr == '\r')
			*ptr = '\n';
	
	CStr255 tmpString = string;
	JavelinErrorBox(tmpString);			

}

//--------------------------------------------------------------------------------------------------
// kAEQuit
//--------------------------------------------------------------------------------------------------
#pragma segment ACommand
pascal void TQuitCmd::IQuitCmd(TDocumentJavelin* itsDoc)	
{
	AEAddressDesc theAddress;

	this->IClientCommand(cRemoteCommand, itsDoc, kCantUndo, kCausesChange, NULL);

	FailInfo fi;
	if (fi.Try())
	{
		itsDoc->GetAEAddressDesc(theAddress);

		TJavelinAppleEvent* aMessage = new TJavelinAppleEvent;		// create the message.
		
		aMessage->IJavelinAppleEvent(kAECmdSuiteID, (AEEventID)kAEQuit, theAddress, kAENoReply | kAEDontReconnect, itsDoc->GetPortRefNum() );
		fMessage = aMessage;
		fi.Success();
	}
	else
	{
		this->Free();
		fi.ReSignal();
	}
}

//--------------------------------------------------------------------------------------------------
// kAEInterval
//--------------------------------------------------------------------------------------------------

#pragma segment ACommand
pascal void TIntervalCmd::IIntervalCmd(TDocumentJavelin* itsDoc)
{
	this->IJavelinClientCommand(itsDoc, kAEInterval, NULL);	

	long theInterval = itsDoc->fInterval;

	fMessage->WriteParameterPtr(keyInterval,typeLongInteger,(Ptr)&theInterval,sizeof(long));
}
//--------------------------------------------------------------------------------------------------
// kAENull
//--------------------------------------------------------------------------------------------------

#pragma segment ACommand
pascal void TWakeupCommand::IWakeupCommand(TDocumentJavelin* itsDoc)
{
	this->IJavelinClientCommand(itsDoc, kAENull, NULL);	

	long clientSessionID = itsDoc->fPortRefNum;

	fMessage->WriteParameterPtr('ssID',typeLongInteger,(Ptr)&clientSessionID,sizeof(long));
}
//--------------------------------------------------------------------------------------------------
// kAEHeartBeat
//--------------------------------------------------------------------------------------------------

#pragma segment ACommand
pascal void THeartBeatCommand::IHeartBeatCommand(TDocumentJavelin* itsDoc)
{
	AEAddressDesc theAddress;

	this->IClientCommand(cRemoteCommand, itsDoc, kCantUndo, kCausesChange, NULL);

	FailInfo fi;
	if (fi.Try())
	{
		itsDoc->GetAEAddressDesc(theAddress);

		TJavelinAppleEvent* aMessage = new TJavelinAppleEvent;		// create the message.
		
		aMessage->IJavelinAppleEvent(kAECmdSuiteID, (AEEventID)kAEHeartBeat, theAddress, kAENoReply, itsDoc->GetPortRefNum() );
		fMessage = aMessage;
		fi.Success();
	}
	else
	{
		this->Free();
		fi.ReSignal();
	}

}

//--------------------------------------------------------------------------------------------------
// kAEVersion
//--------------------------------------------------------------------------------------------------

#pragma segment ACommand
pascal void TJavelinVersionCmd::IJavelinVersionCmd(TDocumentJavelin* itsDoc)
{
	this->IJavelinClientCommand(itsDoc, kAEVersion, NULL);	//guts overridden below
	if (!IsObject(fMessage))
		return;
		
	fMessage->WriteParameterPtr(keyAEVersion, typeChar, (Ptr)gJavelinVersion, strlen(gJavelinVersion));
}

/*
//--------------------------------------------------------------------------------------------------
#pragma segment ACommand
pascal void TJavelinVersionCmd::DoIt()	// override
{
	if (fMessage && ((fMessage->GetSendingMode() & kAEReplyModeMask) == kAEWaitReply))
		this->SendMessage();
} // TJavelinVersionCmd::DoIt
*/
//--------------------------------------------------------------------------------------------------
#pragma segment ACommand
pascal void TJavelinVersionCmd::ProcessReply(TAppleEvent* theReply)
{
	char status[1];
	long actualSize;
	DescType actualCode;
	char theVersion[10];
	CStr255 theVersionString = gEmptyString;	
	inherited::ProcessReply(theReply);
	
	theReply->ReadParameterPtr(keyErrorNumber, typeChar, actualCode,(Ptr) status, sizeof(char), actualSize);

	if (*status != 'S') 
		{
		theReply->ReadParameterPtr(keyAEVersion, typeChar, actualCode,(Ptr) theVersion, 10, actualSize);
		if (theVersion == "0.0")
			JavelinErrorBoxNumber(kNoVersionError);
		else
			JavelinErrorBoxNumber(kBadVersionError);

		((TDocumentJavelin*)fContext)->Disconnect();
		((TDocumentJavelin*)fContext)->CloseAndFree();
		return;
		}
	theReply->ReadParameterPtr(keyAEVersion, typeChar, actualCode,(Ptr) theVersion, 10, actualSize);
		
	theVersion[actualSize] = NULL;

	// Wierdness for converting AE/UNIX strings to Mac strings.
	for (char* ptr = theVersion; *ptr != NULL; ptr++)
		if (*ptr == '\r')
			*ptr = '\n';
	theVersionString = theVersion;
	((TDocumentJavelin*)fContext)->SetHostVersion(theVersionString);
					
}
