//----------------------------------------------------------------------------------------
// UPolevaultApplication.cp
// Copyright 1991-1992 Apple Computer, Inc.
//----------------------------------------------------------------------------------------

#ifndef __UAPPLICATIONPolevault__
#include "UApplicationPolevault.h"
#endif

#ifndef __UDOCUMENTPolevault__
#include "UDocumentPolevault.h"
#endif

#ifndef __UCOMMANDPolevault__
#include "UCommandPolevault.h"
#endif

#ifndef __UREMOTECOMMANDS__
#include "URemoteCommands.h"
#endif

#ifndef __AUXAESuite__
#include "AUXAESuite.h"
#endif

#ifndef TOOLUTILS
#include <ToolUtils.h>
#endif

#ifndef PACKAGES
#include <Packages.h>
#endif

//========================================================================================
// CLASS TApplicationPolevault
//========================================================================================

//----------------------------------------------------------------------------------------
// TApplicationPolevault::IApplicationPolevault: 
//----------------------------------------------------------------------------------------
#pragma segment AInit
pascal void TApplicationPolevault::IApplicationPolevault()
{
	this->IApplication(kSignature,kSignature);
	TIdleBeeper* myIdleBeeper = new TIdleBeeper;
	myIdleBeeper->IIdleBeeper();
	
} // TApplicationPolevault::IApplicationPolevault 


//----------------------------------------------------------------------------------------
// TApplicationPolevault::DoMakeDocument: This method is overridden to return a document
// object of the appropriate class for this application. MacApp calls this method when the
// user chooose New or Open from the File menu. For applications which hande multiple
// document types, the command number can be used to discriminate between different user
// requests. The file object represents the file chosen by the user in the Standard File
// dialog.
//----------------------------------------------------------------------------------------
#pragma segment AOpen		
pascal TDocument* TApplicationPolevault::DoMakeDocument(CommandNumber /*itsCommandNumber*/,
														TFile* itsFile) // Override 
{
	TDocumentPolevault* aDocument = new TDocumentPolevault;
	
	aDocument->IDocumentPolevault(itsFile,kSignature);
	return aDocument;
} // TApplicationPolevault::DoMakeDocument 

//----------------------------------------------------------------------------------------
// TApplicationPolevault::DoMenuCommand: This method is overridden to handle menu items
// which are enabled when this object is in the target chain. The application object is
// always in the target chain.
// 
// The inherited method is called so that MacApp can handle application-level menu items
// like the "About " menu item in the Apple menu.
//----------------------------------------------------------------------------------------
#pragma segment ASelCommand
pascal void TApplicationPolevault::DoMenuCommand(CommandNumber aCommandNumber) // Override 
{
	switch (aCommandNumber) 
	{
		case cCommandHandledByApplication :
			SysBeep(2);
			break;
		default:
			inherited::DoMenuCommand(aCommandNumber);
			break;
	}
} // TApplicationPolevault::DoMenuCommand 

//----------------------------------------------------------------------------------------
// TApplicationPolevault::DoSetupMenus: This method is overridden to enable menu items
// which should be enabled when this object is in the target chain. MacApp initially
// disables all menu items, then lets the objects in the target chain enable those items
// they handle.
//
// The application object is always in the target chain, so the About menu item in the
// Apple menu and the item with command number cMenuHandledByApplication should be enabled
// even if there are no open documents.
// 
// The inherited method is called so that MacApp can enable application-level menu items
// like the "About " menu item.
//----------------------------------------------------------------------------------------
#pragma segment ARes
pascal void TApplicationPolevault::DoSetupMenus() // Override 
{
	inherited::DoSetupMenus();				// Always call the inherited method first
	Enable(cCommandHandledByApplication, !MemSpaceIsLow());

} // TApplicationPolevault::DoSetupMenus 

//----------------------------------------------------------------------------------------
#pragma segment ARes
pascal void TApplicationPolevault::Idle(IdlePhase phase)
{
	inherited::Idle(phase);
	TDocument* doc;
	CDocumentIterator List(this);
	
	for (doc = List.FirstDocument(); List.More(); doc = List.NextDocument())
	{
		doc->DoIdle(idleBegin);		// I wonder what the arg means.
	}

//	return false;		// Returns TRUE if it frees itself so caller can know
}
//--------------------------------------------------------------------------------------------------
#pragma segment ARes
pascal void TApplicationPolevault::DoAppleCommand(CommandNumber aCommandNumber,
												const AppleEvent& message,
												const AppleEvent& reply)  // override
{
	long actualSize;
	DescType actualCode;
	CStr255 theString = gEmptyString;
	long theSessionID = 0;
	TDocumentPolevault* aDoc;	 

switch (aCommandNumber) {
	case cAEHeartBeat:

		FailOSErr(AEGetParamPtr(message, keySessionID, typeLongInteger, actualCode, (Ptr) & theSessionID, sizeof(theSessionID), actualSize));
		aDoc = (TDocumentPolevault*)this->LocateDocument(theSessionID);
		if (aDoc != NULL)
			aDoc->HaveAHeartBeat();

		break;

	default:
		{
		inherited::DoAppleCommand(aCommandNumber, message, reply);
		}
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment ARes
pascal TDocumentPolevault* TApplicationPolevault::LocateDocument(long theSessionID)
{
	CDocumentIterator iter(this);

	for (TDocument* aDocument = iter.FirstDocument(); iter.More(); aDocument = iter.NextDocument())
		if (((TDocumentPolevault*)aDocument)->LocateDocument(theSessionID))
			return (TDocumentPolevault*) aDocument;
	return NULL;								// No document already open
}
//--------------------------------------------------------------------------------------------------
#pragma segment ARes
pascal Boolean TApplicationPolevault::IsThisMyDoc(TDocument* theDoc)
{
	CDocumentIterator iter(this);
	if (IsObject(theDoc) && (iter.FirstDocument() != NULL))
		{
		if (GetClassID(theDoc) == GetClassID(iter.FirstDocument()))
			{
			for (TDocument* aDocument = iter.FirstDocument(); iter.More(); aDocument = iter.NextDocument())
				if (aDocument == theDoc)   //compare pointers right?
					return TRUE;
			}
		}
		
	return FALSE;								// No document already open
}

//--------------------------------------------------------------------------------------------------
#pragma segment ANonRes
pascal void ptocstr(char* string)
{
	register char *to = string;
	register char length = *string++, i;

	for (i = 1; i <= length; i++)
		*to++ = *string++;
	*to = '\0';
}

//==================================================================================================
#pragma segment AIdle

void pascal TCohandler::Free()
/*
Task:
	Remove me as Cohandler before freeing.
*/
{
	// remove as cohandler of application object
	gApplication->InstallCohandler(this, false);

	inherited::Free();
}
//--------------------------------------------------------------------------------------------------
#pragma segment AIdle

void pascal TCohandler::ICohandler()
{
	// install as cohandler of application object
	this->IEventHandler(NULL);
	gApplication->InstallCohandler(this, true);
}
//==================================================================================================
#pragma segment AIdle

void pascal TIdleBeeper::IIdleBeeper()
{
	this->ICohandler();
	fIdleFreq = 1800;   //30 seconds minimum
	fLastHeartBeat = TickCount()/60;
}

//--------------------------------------------------------------------------------------------------
#pragma segment AIdle

Boolean pascal TIdleBeeper::DoIdle(IdlePhase)
{
	if (((TickCount()/60) - fLastHeartBeat) > 30)
		{
		fLastHeartBeat = TickCount()/60;

		CDocumentIterator iter(gApplication);

		for (TDocument* aDocument = iter.FirstDocument(); iter.More(); aDocument = iter.NextDocument())
			{
			for (int i = 0; i < ((TDocumentPolevault*)aDocument)->nAddrs; i++)
			{
			THeartBeatCommand* aCommand = new THeartBeatCommand;
			aCommand->IHeartBeatCommand((TDocumentPolevault*)aDocument, i);
 			((TDocumentPolevault*)aDocument)->PostCommand(aCommand);
			}
//			THeartBeatCommand* aCommand = new THeartBeatCommand;
//			aCommand->IHeartBeatCommand((TDocumentPolevault*)aDocument);
// 			((TDocumentPolevault*)aDocument)->PostCommand(aCommand);
			if (! ((TDocumentPolevault*)aDocument)->CheckPulse())            //he's dead Jim!
				((TDocumentPolevault*)aDocument)->CloseAndFree();
			}
		}

	return false;  //doesn't matter I guess
}
