#
#	MacMerge.make
#
# Macintosh MPW Makefile for the Eclectus Merge utility
# Copyright (C) 1992-96 Eclectus (D. John Anderson, Alan B. Harper).

# This file is part of the Eclectus integration utilities.

# Eclectus integration utilities are free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 1, or
# (at your option) any later version.

# Eclectus integration utilities are distributed in the hope that they
# will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the Eclectus integration utilities; see the file COPYING.
# If not, write to the Free Software Foundation, 675 Mass Ave, Cambridge,
# MA 02139, USA.

# This make file has the following targets
#
#		Clean:						Remove all derived files, create directories necessary for other targets below.
#
#		All/<no target>:	Merge and MergeDebug as fat tools*
#		Release:					Fat Merge
#		PPC:							PPC only Merge
#		68K:							68K only Merge
#		Debug							Fat MergeDebug*
#		DebugPPC:					PPC only MergeDebug*
#		Debug68K:					68K only MergeDebug*
#
#		Install:					Install fat Merge and MergeDebug*
#		InstallRelease:		Install fat Merge*
#		InstallPPC:				Install PPC Merge*
#		Install68K:				Install 68K Merge*
#		InstallDebug:			Install fat MergeDebug*
#		InstallDebugPPC:	Install PPC only MergeDebug*
#		InstallDebug68K:	Install 68K only MergeDebug*
#
#		Test:							Test Merge
#		TestDebug:				Test MergeDebug*
#
#	*	The code for MergeDebug is not included in the standard distribution, since it is used
#		only to test the code. Write Alan Harper (aharper@dnai.com) if you have a need for this code.
#		Also, the Install... targets place the tools in {MPW}ECTools, which may not be where you
#		want them. Change BIN_DIR to adjust this.

MWCPPC_OPTIONS = 
	-d DF_MACHINE_MACINTOSH 
	-d DF_COMPILER_MW 
	-d DF_CHIP_PPC 
	-mapcr 
	-maxerrors 10 
	-once 
	-opt all 
	-proto strict 
	-r 
	-str readonly 
	-trigraphs on 
	-w full


MWC68K_OPTIONS = 
	-d DF_MACHINE_MACINTOSH 
	-d DF_COMPILER_MW 
	-d DF_CHIP_68K 
	-enum int 
	-mapcr 
	-maxerrors 10 
	-mbg off 
	-mc68020 
	-model codesmart 
	-once 
	-opt all 
	-proto strict 
	-r 
	-str pool 
	-sym off 
	-trigraphs on 
	-w full

ADDL_MWCPPC_OPTIONS = 

ADDL_MWC68K_OPTIONS = 

MWLINKPPC_OPTIONS = 
	-d 
	-c 'MPS ' 
	-t 'MPST'

MWLINK68K_OPTIONS = -mpwtool

MWLINKPPC_LIBS = 
	"{MWPPCLibraries}"MWMPWCRuntime.Lib 
	"{MWPPCLibraries}"'MPW ANSI.C.PPC.Lib' 
	"{MWPPCLibraries}"PPCToolLibs.o 
	"{SharedLibraries}"InterfaceLib 
	"{SharedLibraries}"MathLib 

MWLINK68K_LIBS = 
	"{MW68KLibraries}"MPWRuntime.68K.lib 
	"{MW68KLibraries}"'MPW ANSI (4i/8d) C.68K.Lib' 
	"{MW68KLibraries}"'MathLib68K (4i/8d).Lib' 
	"{MW68KLibraries}"ToolLibs.o 
	"{MW68KLibraries}"MacOS.Lib 

DERIVED_DIR = :MergeR.i

DERIVED_DEBUG_DIR = :MergeD.i

TOOL_NAME = Merge

DEBUG_TOOL_NAME = MergeDebug

INSTALL_AS_NAME = Merge

INSTALL_FROM_NAME = Merge

BIN_DIR = {MPW}ECTools

LOCAL_BIN_DIR = :MacTools

MERGE_TO_OBERISH_DIR = ::Oberish

H_FILES = 
	diff.h

MERGE_PPC_OBJS = 
	{DERIVED_DIR}:analyze.c.x 
	{DERIVED_DIR}:io.c.x 
	{DERIVED_DIR}:util.c.x 
	{DERIVED_DIR}:Merge.c.x 
	{DEBUG_PPC_OBJS}

MERGE_68K_OBJS = 
	{DERIVED_DIR}:analyze.c.o 
	{DERIVED_DIR}:io.c.o 
	{DERIVED_DIR}:util.c.o 
	{DERIVED_DIR}:Merge.c.o 
	{DEBUG_68K_OBJS}

DEBUG_PPC_OBJS =

DEBUG_68K_OBJS =

Release  Merge

All  Release Debug

PPC  Merge.PPC

68K  Merge.68K

Debug  MergeDebug

DebugPPC  MergeDebug.PPC

Debug68K  MergeDebug.68K

Merge  {DERIVED_DIR}:Merge

Merge.PPC  {DERIVED_DIR}:Merge.PPC

Merge.68K  {DERIVED_DIR}:Merge.68K

MergeDebug  
	Make -f MacMerge.make 
		-d DEBUG_PPC_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.x" 
		-d DEBUG_68K_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.o" 
		-d DERIVED_DIR="{DERIVED_DEBUG_DIR}" 
		-d ADDL_MWCPPC_OPTIONS="-d DF_DEBUG -i {MERGE_TO_OBERISH_DIR}" 
		-d ADDL_MWC68K_OPTIONS="-d DF_DEBUG -i {MERGE_TO_OBERISH_DIR}" 
		-d TOOL_NAME={DEBUG_TOOL_NAME} 
		{DERIVED_DEBUG_DIR}:{DEBUG_TOOL_NAME}

MergeDebug.PPC 
	Make -f MacMerge.make 
		-d DEBUG_PPC_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.x" 
		-d DERIVED_DIR="{DERIVED_DEBUG_DIR}" 
		-d ADDL_MWCPPC_OPTIONS="-d DF_DEBUG -i {MERGE_TO_OBERISH_DIR}" 
		-d TOOL_NAME={DEBUG_TOOL_NAME} 
		{DERIVED_DEBUG_DIR}:{DEBUG_TOOL_NAME}.PPC

MergeDebug.68K 
	Make -f MacMerge.make 
		-d DEBUG_68K_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.o" 
		-d DERIVED_DIR="{DERIVED_DEBUG_DIR}" 
		-d ADDL_MWC68K_OPTIONS="-d DF_DEBUG -i {MERGE_TO_OBERISH_DIR}" 
		-d TOOL_NAME={DEBUG_TOOL_NAME} 
		{DERIVED_DEBUG_DIR}:{DEBUG_TOOL_NAME}.68K

{MERGE_PPC_OBJS}  {H_FILES}

{MERGE_68K_OBJS}  {H_FILES}

{DERIVED_DIR}:  :

{DERIVED_DIR}:  {MERGE_TO_OBERISH_DIR}:

{DERIVED_DIR}:{TOOL_NAME}  {DERIVED_DIR}:{TOOL_NAME}.PPC {DERIVED_DIR}:{TOOL_NAME}.68K
	Duplicate -y {DERIVED_DIR}:{TOOL_NAME}.PPC {DERIVED_DIR}:{TOOL_NAME}
	Rez -append -o {DERIVED_DIR}:{TOOL_NAME} -d RSRCNAME=""{DERIVED_DIR}:{TOOL_NAME}.68K"" MacCopyRSRC.r

{DERIVED_DIR}:{TOOL_NAME}.PPC   {MERGE_PPC_OBJS}
	MWLinkPPC {MWLINKPPC_LIBS} {MERGE_PPC_OBJS} {MWLINKPPC_OPTIONS} -o {Targ}
	Rez -d APPNAME="{TOOL_NAME}" MacGenericCFRG.r -o {Targ}

{DERIVED_DIR}:{TOOL_NAME}.68K   {MERGE_68K_OBJS}
	MWLink68K {MWLINK68K_LIBS} {MERGE_68K_OBJS} {MWLINK68K_OPTIONS} -o {Targ}

Install  InstallRelease InstallDebug

InstallRelease  {DERIVED_DIR}:{INSTALL_FROM_NAME}
	If not `Exists {BIN_DIR}:`
		NewFolder {BIN_DIR}:
	End
	Duplicate -y {DERIVED_DIR}:{INSTALL_FROM_NAME} {BIN_DIR}:{INSTALL_AS_NAME}
	If not `Exists {LOCAL_BIN_DIR}:`
		NewFolder {LOCAL_BIN_DIR}:
	End
	Duplicate -y {DERIVED_DIR}:{INSTALL_FROM_NAME} {LOCAL_BIN_DIR}:{INSTALL_AS_NAME}.mpw

InstallPPC 
	Make -f MacMerge.make InstallRelease 
		-d INSTALL_FROM_NAME="Merge.PPC" 
		-d TOOL_NAME="Merge"

Install68K 
	Make -f MacMerge.make InstallRelease 
		-d INSTALL_FROM_NAME="Merge.68K" 
		-d TOOL_NAME="Merge"

InstallDebug 
	Make -f MacMerge.make InstallRelease 
		-d ADDL_MWCPPC_OPTIONS="-d DF_DEBUG -i {MERGE_TO_OBERISH_DIR}" 
		-d ADDL_MWC68K_OPTIONS="-d DF_DEBUG -i {MERGE_TO_OBERISH_DIR}" 
		-d DEBUG_PPC_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.x" 
		-d DEBUG_68K_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.o" 
		-d DERIVED_DIR="{DERIVED_DEBUG_DIR}" 
		-d INSTALL_FROM_NAME="MergeDebug" 
		-d INSTALL_AS_NAME="MergeDebug" 
		-d TOOL_NAME="MergeDebug"

InstallDebugPPC 
	Make -f MacMerge.make InstallRelease 
		-d ADDL_MWCPPC_OPTIONS="-d DF_DEBUG -i {MERGE_TO_OBERISH_DIR}" 
		-d DEBUG_PPC_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.x" 
		-d DERIVED_DIR="{DERIVED_DEBUG_DIR}" 
		-d INSTALL_FROM_NAME="MergeDebug.PPC" 
		-d INSTALL_AS_NAME="MergeDebug" 
		-d TOOL_NAME="MergeDebug"

InstallDebug68K 
	Make -f MacMerge.make InstallRelease 
		-d ADDL_MWC68K_OPTIONS="-d DF_DEBUG -i {MERGE_TO_OBERISH_DIR}" 
		-d DEBUG_68K_OBJS="{DERIVED_DEBUG_DIR}:DebugMalloc.c.o" 
		-d DERIVED_DIR="{DERIVED_DEBUG_DIR}" 
		-d INSTALL_FROM_NAME="MergeDebug.68K" 
		-d INSTALL_AS_NAME="MergeDebug" 
		-d TOOL_NAME="MergeDebug"

Clean 
	Delete -y -i {DERIVED_DIR}
	NewFolder {DERIVED_DIR}
	Delete -y -i {DERIVED_DEBUG_DIR}
	NewFolder {DERIVED_DEBUG_DIR}

Test 
	Merge testfile0 testfile1 testfile2 testfile3 testfile4 {DERIVED_DIR}:testfile.out > Dev:Stderr
	compare {DERIVED_DIR}:!testfile.out testfile.merge > Dev:Stderr
	Delete -i -y {DERIVED_DIR}:testd.out
	Merge :testd0 :testd1 :testd2 :testd3 :testd4 {DERIVED_DIR}:testd.out > Dev:Stderr
	Difference {DERIVED_DIR}:testd.out :testd.mac.merge >{DERIVED_DIR}:testd.check

TestDebug 
	MergeDebug -x -X testfile0 testfile1 testfile2 testfile3 testfile4 {DERIVED_DIR}:testfile.out > Dev:Stderr
	Compare {DERIVED_DIR}:!testfile.out testfile.merge > Dev:Stderr
	Delete -i -y {DERIVED_DIR}:testd.out
	MergeDebug -x -X :testd0 :testd1 :testd2 :testd3 :testd4 {DERIVED_DIR}:testd.out > Dev:Stderr
	DifferenceDebug {DERIVED_DIR}:testd.out :testd.mac.merge >{DERIVED_DIR}:testd.check

.c.x  .c
	MWCPPC {DepDir}{Default}.c {MWCPPC_OPTIONS} {ADDL_MWCPPC_OPTIONS} -o {Targ}

.c.o  .c
	MWC68K {DepDir}{Default}.c {MWC68K_OPTIONS} {ADDL_MWC68K_OPTIONS} -o {Targ}
